{% extends "wagtailadmin/base.html" %}
{% load i18n %}
{% block titletag %}{% blocktrans with title=page.title%}Unpublish {{ title }}{% endblocktrans %}{% endblock %}
{% block content %}
    {% trans "Unpublish" as unpublish_str %}
    {% include "wagtailadmin/shared/header.html" with title=unpublish_str subtitle=page.title icon="doc-empty-inverse" %}

    <div class="nice-padding">
        <p>{% trans "Are you sure you want to unpublish this page?" %}</p>
        <form action="{% url 'wagtailadmin_pages:unpublish' page.id %}" method="POST">
            {% csrf_token %}
            <input type="submit" value="{% trans 'Yes, unpublish it' %}">
        </form>
    </div>
{% endblock %}
